@@ -1,10 +1,10 @@
-import io
-import optparse
 import os
 import platform
 import re
 import sys
 
+import flake8
+from flake8.options.manager import OptionManager
 import pep8ext_naming
 
 
@@ -25,7 +25,7 @@ def main():
     errors = 0
     for filename in os.listdir('testsuite'):
         filepath = os.path.join('testsuite', filename)
-        with io.open(filepath, encoding='utf8') as fd:
+        with open(filepath, encoding='utf8') as fd:
             lines = list(fd)
             if not is_test_allowed(lines):
                 continue
@@ -70,18 +70,29 @@ def load_tests(lines):
         yield testcase, code, options
 
 
-class OptionsManager(optparse.OptionParser):
-    """A Flake8-2.x-compatible OptionsManager."""
-    def __init__(self, *args, **kwargs):
-        optparse.OptionParser.__init__(self, *args, **kwargs)
-        self.config_options = []
-
-
 def parse_options(checker, options):
     """Parse the CLI-style flags from `options` and expose to `checker`"""
-    options_manager = OptionsManager('flake8')
+    extra_kwargs = {}
+    if flake8.__version_info__[0] >= 6:
+        extra_kwargs = {
+            "formatter_names": []
+        }
+
+    options_manager = OptionManager(
+        version=flake8.__version__,
+        plugin_versions=f"naming: {pep8ext_naming.__version__}",
+        parents=[],
+        **extra_kwargs,
+    )
+    options_manager.add_option('--select', default=[])
+    options_manager.add_option('--extended-default-select', default=['N'])
+    options_manager.add_option('--extend-select', default=['N'])
+    options_manager.add_option('--ignore', default=[])
+    options_manager.add_option('--extend-ignore', default=[])
+    options_manager.add_option('--enable-extensions', default=[])
+    options_manager.add_option('--extended-default-ignore', default=[])
     checker.add_options(options_manager)
-    processed_options, _ = options_manager.parse_args(options)
+    processed_options = options_manager.parse_args(options)
     checker.parse_options(processed_options)
 
 
