@@ -1,94 +1,66 @@
-# -*- coding: utf-8 -*-
 """Checker of PEP-8 Naming Conventions."""
-import sys
+import ast
+from ast import iter_child_nodes
 from collections import deque
-from fnmatch import fnmatch
+from collections.abc import Iterable
+from fnmatch import fnmatchcase
 from functools import partial
 from itertools import chain
 
-from flake8_polyfill import options
+from flake8 import style_guide
 
-try:
-    import ast
-    from ast import iter_child_nodes
-except ImportError:
-    from flake8.util import ast, iter_child_nodes
-
-__version__ = '0.10.0'
-
-PYTHON_VERSION = sys.version_info[:3]
-PY2 = PYTHON_VERSION[0] == 2
+__version__ = '0.14.1'
 
+CLASS_METHODS = frozenset((
+    '__new__',
+    '__init_subclass__',
+    '__class_getitem__',
+))
 METACLASS_BASES = frozenset(('type', 'ABCMeta'))
 
 # Node types which may contain class methods
-METHOD_CONTAINER_NODES = {ast.If, ast.While, ast.For, ast.With}
-FUNC_NODES = (ast.FunctionDef,)
-
-if PY2:
-    METHOD_CONTAINER_NODES |= {ast.TryExcept, ast.TryFinally}
-else:
-    METHOD_CONTAINER_NODES |= {ast.Try}
-
-if PYTHON_VERSION > (3, 5):
-    FUNC_NODES += (ast.AsyncFunctionDef,)
-    METHOD_CONTAINER_NODES |= {ast.AsyncWith, ast.AsyncFor}
-
-if PY2:
-    def _unpack_args(args):
-        ret = []
-        for arg in args:
-            if isinstance(arg, ast.Tuple):
-                ret.extend(_unpack_args(arg.elts))
-            else:
-                ret.append((arg, arg.id))
-        return ret
-
-    def get_arg_name_tuples(node):
-        return _unpack_args(node.args.args)
-elif PYTHON_VERSION < (3, 8):
-    def get_arg_name_tuples(node):
-        groups = (node.args.args, node.args.kwonlyargs)
-        return [(arg, arg.arg) for args in groups for arg in args]
-else:
-    def get_arg_name_tuples(node):
-        groups = (node.args.posonlyargs, node.args.args, node.args.kwonlyargs)
-        return [(arg, arg.arg) for args in groups for arg in args]
+METHOD_CONTAINER_NODES = {
+    ast.If,
+    ast.While,
+    ast.For,
+    ast.With,
+    ast.Try,
+    ast.AsyncWith,
+    ast.AsyncFor,
+}
+FUNC_NODES = (ast.FunctionDef, ast.AsyncFunctionDef)
 
 
 class _ASTCheckMeta(type):
     def __init__(cls, class_name, bases, namespace):
+        cls.codes = tuple(code for code in namespace if code.startswith('N'))
         try:
-            cls._checks.append(cls())
+            cls.all.append(cls())
         except AttributeError:
-            cls._checks = []
+            cls.all = []
 
 
 def _err(self, node, code, **kwargs):
     lineno, col_offset = node.lineno, node.col_offset
     if isinstance(node, ast.ClassDef):
-        if PYTHON_VERSION < (3, 8):
-            lineno += len(node.decorator_list)
         col_offset += 6
     elif isinstance(node, FUNC_NODES):
-        if PYTHON_VERSION < (3, 8):
-            lineno += len(node.decorator_list)
         col_offset += 4
     code_str = getattr(self, code)
     if kwargs:
         code_str = code_str.format(**kwargs)
-    return lineno, col_offset + 1, '%s %s' % (code, code_str), self
+    return lineno, col_offset + 1, f'{code} {code_str}', self
 
 
 def _ignored(name, ignore):
-    return any(fnmatch(name, i) for i in ignore)
+    return any(fnmatchcase(name, i) for i in ignore)
 
 
 BaseASTCheck = _ASTCheckMeta('BaseASTCheck', (object,),
                              {'__doc__': "Base for AST Checks.", 'err': _err})
 
 
-class _FunctionType(object):
+class _FunctionType:
     CLASSMETHOD = 'classmethod'
     STATICMETHOD = 'staticmethod'
     FUNCTION = 'function'
@@ -100,6 +72,10 @@ _default_ignore_names = [
         'tearDown',
         'setUpClass',
         'tearDownClass',
+        'setUpModule',
+        'tearDownModule',
+        'asyncSetUp',
+        'asyncTearDown',
         'setUpTestData',
         'failureException',
         'longMessage',
@@ -117,49 +93,47 @@ def _build_decorator_to_type(classmethod_decorators, staticmethod_decorators):
     return decorator_to_type
 
 
-class NamingChecker(object):
+class NamingChecker:
     """Checker of PEP-8 Naming Conventions."""
     name = 'naming'
     version = __version__
+    visitors = BaseASTCheck.all
     decorator_to_type = _build_decorator_to_type(
         _default_classmethod_decorators, _default_staticmethod_decorators)
     ignore_names = frozenset(_default_ignore_names)
 
     def __init__(self, tree, filename):
-        self.visitors = BaseASTCheck._checks
-        self.parents = deque()
-        self._node = tree
+        self.tree = tree
 
     @classmethod
     def add_options(cls, parser):
-        options.register(parser, '--ignore-names',
-                         default=_default_ignore_names,
-                         action='store',
-                         type='string',
-                         parse_from_config=True,
-                         comma_separated_list=True,
-                         help='List of names or glob patterns the pep8-naming '
-                              'plugin should ignore. (Defaults to %default)')
-
-        options.register(parser, '--classmethod-decorators',
-                         default=_default_classmethod_decorators,
-                         action='store',
-                         type='string',
-                         parse_from_config=True,
-                         comma_separated_list=True,
-                         help='List of method decorators pep8-naming plugin '
-                              'should consider classmethods (Defaults to '
-                              '%default)')
-
-        options.register(parser, '--staticmethod-decorators',
-                         default=_default_staticmethod_decorators,
-                         action='store',
-                         type='string',
-                         parse_from_config=True,
-                         comma_separated_list=True,
-                         help='List of method decorators pep8-naming plugin '
-                              'should consider staticmethods (Defaults to '
-                              '%default)')
+        parser.add_option(
+            '--ignore-names',
+            default=_default_ignore_names,
+            parse_from_config=True,
+            comma_separated_list=True,
+            help='List of names or glob patterns the pep8-naming '
+            'plugin should ignore. (Defaults to %(default)s)',
+        )
+        parser.add_option(
+            '--classmethod-decorators',
+            default=_default_classmethod_decorators,
+            parse_from_config=True,
+            comma_separated_list=True,
+            help='List of method decorators pep8-naming plugin '
+            'should consider classmethods (Defaults to '
+            '%(default)s)',
+        )
+        parser.add_option(
+            '--staticmethod-decorators',
+            default=_default_staticmethod_decorators,
+            parse_from_config=True,
+            comma_separated_list=True,
+            help='List of method decorators pep8-naming plugin '
+            'should consider staticmethods (Defaults to '
+            '%(default)s)',
+        )
+        parser.extend_default_ignore(['N818'])
 
     @classmethod
     def parse_options(cls, options):
@@ -168,33 +142,38 @@ class NamingChecker(object):
             options.classmethod_decorators,
             options.staticmethod_decorators)
 
+        # Build a list of node visitors based the error codes that have been
+        # selected in the style guide. Only the checks that have been selected
+        # will be evaluated as a performance optimization.
+        engine = style_guide.DecisionEngine(options)
+        cls.visitors = frozenset(
+            visitor for visitor in BaseASTCheck.all for code in visitor.codes
+            if engine.decision_for(code) is style_guide.Decision.Selected
+        )
+
     def run(self):
-        return self.visit_tree(self._node) if self._node else ()
+        return self.visit_tree(self.tree, deque()) if self.tree else ()
 
-    def visit_tree(self, node):
-        for error in self.visit_node(node):
-            yield error
-        self.parents.append(node)
+    def visit_tree(self, node, parents: deque):
+        yield from self.visit_node(node, parents)
+        parents.append(node)
         for child in iter_child_nodes(node):
-            for error in self.visit_tree(child):
-                yield error
-        self.parents.pop()
+            yield from self.visit_tree(child, parents)
+        parents.pop()
 
-    def visit_node(self, node):
+    def visit_node(self, node, parents: Iterable):
         if isinstance(node, ast.ClassDef):
             self.tag_class_functions(node)
         elif isinstance(node, FUNC_NODES):
             self.find_global_defs(node)
 
         method = 'visit_' + node.__class__.__name__.lower()
-        parents = self.parents
         ignore_names = self.ignore_names
         for visitor in self.visitors:
             visitor_method = getattr(visitor, method, None)
             if visitor_method is None:
                 continue
-            for error in visitor_method(node, parents, ignore_names):
-                yield error
+            yield from visitor_method(node, parents, ignore_names)
 
     def tag_class_functions(self, cls_node):
         """Tag functions if they are methods, classmethods, staticmethods"""
@@ -214,7 +193,7 @@ class NamingChecker(object):
                 late_decoration[meth.id] = self.decorator_to_type[func_name]
 
         # If this class inherits from a known metaclass base class, it is
-        # itself a metaclass, and we'll consider all of it's methods to be
+        # itself a metaclass, and we'll consider all of its methods to be
         # classmethods.
         bases = chain(
             (b.id for b in cls_node.bases if isinstance(b, ast.Name)),
@@ -234,17 +213,25 @@ class NamingChecker(object):
             if not isinstance(node, FUNC_NODES):
                 continue
             node.function_type = _FunctionType.METHOD
-            if node.name in ('__new__', '__init_subclass__') or ismetaclass:
+            if node.name in CLASS_METHODS or ismetaclass:
                 node.function_type = _FunctionType.CLASSMETHOD
             if node.name in late_decoration:
                 node.function_type = late_decoration[node.name]
             elif node.decorator_list:
-                names = [self.decorator_to_type[d.id]
-                         for d in node.decorator_list
-                         if isinstance(d, ast.Name) and
-                         d.id in self.decorator_to_type]
-                if names:
-                    node.function_type = names[0]
+                for d in node.decorator_list:
+                    name = self.find_decorator_name(d)
+                    if name in self.decorator_to_type:
+                        node.function_type = self.decorator_to_type[name]
+                        break
+
+    @classmethod
+    def find_decorator_name(cls, d):
+        if isinstance(d, ast.Name):
+            return d.id
+        elif isinstance(d, ast.Attribute):
+            return d.attr
+        elif isinstance(d, ast.Call):
+            return cls.find_decorator_name(d.func)
 
     @staticmethod
     def find_global_defs(func_def_node):
@@ -267,14 +254,37 @@ class ClassNameCheck(BaseASTCheck):
     Classes for internal use have a leading underscore in addition.
     """
     N801 = "class name '{name}' should use CapWords convention"
+    N818 = "exception name '{name}' should be named with an Error suffix"
 
-    def visit_classdef(self, node, parents, ignore=None):
+    @classmethod
+    def get_classdef(cls, name, parents: Iterable):
+        for parent in parents:
+            for node in parent.body:
+                if isinstance(node, ast.ClassDef) and node.name == name:
+                    return node
+
+    @classmethod
+    def superclass_names(cls, name, parents: Iterable, _names=None):
+        names = _names or set()
+        classdef = cls.get_classdef(name, parents)
+        if not classdef:
+            return names
+        for base in classdef.bases:
+            if isinstance(base, ast.Name) and base.id not in names:
+                names.add(base.id)
+                names.update(cls.superclass_names(base.id, parents, names))
+        return names
+
+    def visit_classdef(self, node, parents: Iterable, ignore=None):
         name = node.name
         if _ignored(name, ignore):
             return
         name = name.strip('_')
         if not name[:1].isupper() or '_' in name:
             yield self.err(node, 'N801', name=name)
+        superclasses = self.superclass_names(name, parents)
+        if "Exception" in superclasses and not name.endswith("Error"):
+            yield self.err(node, 'N818', name=name)
 
 
 class FunctionNameCheck(BaseASTCheck):
@@ -290,13 +300,26 @@ class FunctionNameCheck(BaseASTCheck):
     N802 = "function name '{name}' should be lowercase"
     N807 = "function name '{name}' should not start and end with '__'"
 
-    def visit_functiondef(self, node, parents, ignore=None):
+    @staticmethod
+    def has_override_decorator(node):
+        for d in node.decorator_list:
+            if isinstance(d, ast.Name) and d.id == 'override':
+                return True
+            if (isinstance(d, ast.Attribute) and isinstance(d.value, ast.Name)
+                    and d.value.id == 'typing' and d.attr == 'override'):
+                return True
+        return False
+
+    def visit_functiondef(self, node, parents: Iterable, ignore=None):
         function_type = getattr(node, 'function_type', _FunctionType.FUNCTION)
         name = node.name
         if _ignored(name, ignore):
             return
         if name in ('__dir__', '__getattr__'):
             return
+        if (function_type != _FunctionType.FUNCTION
+                and self.has_override_decorator(node)):
+            return
         if name.lower() != name:
             yield self.err(node, 'N802', name=name)
         if (function_type == _FunctionType.FUNCTION
@@ -318,37 +341,37 @@ class FunctionArgNamesCheck(BaseASTCheck):
     N804 = "first argument of a classmethod should be named 'cls'"
     N805 = "first argument of a method should be named 'self'"
 
-    def visit_functiondef(self, node, parents, ignore=None):
+    def visit_functiondef(self, node, parents: Iterable, ignore=None):
+        args = node.args.posonlyargs + node.args.args + node.args.kwonlyargs
 
-        def arg_name(arg):
-            try:
-                return arg, arg.arg
-            except AttributeError:  # PY2
-                return node, arg
+        # Start by applying checks that are specific to the first argument.
+        #
+        # Note: The `ignore` check shouldn't be necessary here because we'd
+        #       expect users to explicitly ignore N804/N805 when using names
+        #       other than `self` and `cls` rather than ignoring names like
+        #       `klass` to get around these checks. However, a previous
+        #       implementation allowed for that, so we retain that behavior
+        #       for backwards compatibility.
+        if args and (name := args[0].arg) and not _ignored(name, ignore):
+            function_type = getattr(node, 'function_type', None)
+            if function_type == _FunctionType.METHOD and name != 'self':
+                yield self.err(args[0], 'N805')
+            elif function_type == _FunctionType.CLASSMETHOD and name != 'cls':
+                yield self.err(args[0], 'N804')
 
-        for arg, name in arg_name(node.args.vararg), arg_name(node.args.kwarg):
-            if name is None or _ignored(name, ignore):
-                continue
-            if name.lower() != name:
-                yield self.err(arg, 'N803', name=name)
-                return
+        # Also add the special *arg and **kwarg arguments for the rest of the
+        # checks when they're present. We didn't include them above because
+        # the "first argument" naming checks shouldn't be applied to them
+        # when they're the function's only argument(s).
+        if node.args.vararg:
+            args.append(node.args.vararg)
+        if node.args.kwarg:
+            args.append(node.args.kwarg)
 
-        arg_name_tuples = get_arg_name_tuples(node)
-        if not arg_name_tuples:
-            return
-        arg0, name0 = arg_name_tuples[0]
-        function_type = getattr(node, 'function_type', _FunctionType.FUNCTION)
-
-        if function_type == _FunctionType.METHOD:
-            if name0 != 'self' and not _ignored(name0, ignore):
-                yield self.err(arg0, 'N805')
-        elif function_type == _FunctionType.CLASSMETHOD:
-            if name0 != 'cls' and not _ignored(name0, ignore):
-                yield self.err(arg0, 'N804')
-        for arg, name in arg_name_tuples:
+        for arg in args:
+            name = arg.arg
             if name.lower() != name and not _ignored(name, ignore):
                 yield self.err(arg, 'N803', name=name)
-                return
 
     visit_asyncfunctiondef = visit_functiondef
 
@@ -363,7 +386,7 @@ class ImportAsCheck(BaseASTCheck):
     N814 = "camelcase '{name}' imported as constant '{asname}'"
     N817 = "camelcase '{name}' imported as acronym '{asname}'"
 
-    def visit_importfrom(self, node, parents, ignore=None):
+    def visit_importfrom(self, node, parents: Iterable, ignore=None):
         for name in node.names:
             asname = name.asname
             if not asname:
@@ -395,7 +418,7 @@ class VariablesCheck(BaseASTCheck):
     N815 = "variable '{name}' in class scope should not be mixedCase"
     N816 = "variable '{name}' in global scope should not be mixedCase"
 
-    def _find_errors(self, assignment_target, parents, ignore):
+    def _find_errors(self, assignment_target, parents: Iterable, ignore):
         for parent_func in reversed(parents):
             if isinstance(parent_func, ast.ClassDef):
                 checker = self.class_variable_check
@@ -423,49 +446,38 @@ class VariablesCheck(BaseASTCheck):
                     return True
         return False
 
-    def visit_assign(self, node, parents, ignore=None):
+    def visit_assign(self, node, parents: Iterable, ignore=None):
         if self.is_namedtupe(node.value):
             return
         for target in node.targets:
-            for error in self._find_errors(target, parents, ignore):
-                yield error
+            yield from self._find_errors(target, parents, ignore)
 
-    def visit_namedexpr(self, node, parents, ignore):
+    def visit_namedexpr(self, node, parents: Iterable, ignore):
         if self.is_namedtupe(node.value):
             return
-        for error in self._find_errors(node.target, parents, ignore):
-            yield error
+        yield from self._find_errors(node.target, parents, ignore)
 
     visit_annassign = visit_namedexpr
 
-    def visit_with(self, node, parents, ignore):
-        if PY2:
-            for error in self._find_errors(
-                    node.optional_vars, parents, ignore):
-                yield error
-            return
+    def visit_with(self, node, parents: Iterable, ignore):
         for item in node.items:
-            for error in self._find_errors(
-                    item.optional_vars, parents, ignore):
-                yield error
+            yield from self._find_errors(
+                    item.optional_vars, parents, ignore)
 
     visit_asyncwith = visit_with
 
-    def visit_for(self, node, parents, ignore):
-        for error in self._find_errors(node.target, parents, ignore):
-            yield error
+    def visit_for(self, node, parents: Iterable, ignore):
+        yield from self._find_errors(node.target, parents, ignore)
 
     visit_asyncfor = visit_for
 
-    def visit_excepthandler(self, node, parents, ignore):
+    def visit_excepthandler(self, node, parents: Iterable, ignore):
         if node.name:
-            for error in self._find_errors(node, parents, ignore):
-                yield error
+            yield from self._find_errors(node, parents, ignore)
 
-    def visit_generatorexp(self, node, parents, ignore):
+    def visit_generatorexp(self, node, parents: Iterable, ignore):
         for gen in node.generators:
-            for error in self._find_errors(gen.target, parents, ignore):
-                yield error
+            yield from self._find_errors(gen.target, parents, ignore)
 
     visit_listcomp = visit_dictcomp = visit_setcomp = visit_generatorexp
 
@@ -499,25 +511,11 @@ def _extract_names(assignment_target):
             if element_type is ast.Name:
                 yield element.id
             elif element_type in (ast.Tuple, ast.List):
-                for n in _extract_names(element):
-                    yield n
-            elif not PY2 and element_type is ast.Starred:  # PEP 3132
-                for n in _extract_names(element.value):
-                    yield n
+                yield from _extract_names(element)
+            elif element_type is ast.Starred:  # PEP 3132
+                yield from _extract_names(element.value)
     elif target_type is ast.ExceptHandler:
-        if PY2:
-            # Python 2 supports unpacking tuple exception values.
-            if isinstance(assignment_target.name, ast.Tuple):
-                for name in assignment_target.name.elts:
-                    yield name.id
-            elif isinstance(assignment_target.name, ast.Attribute):
-                # Python 2 also supports assigning an exception to an attribute
-                # eg. except Exception as obj.attr
-                yield assignment_target.name.attr
-            else:
-                yield assignment_target.name.id
-        else:
-            yield assignment_target.name
+        yield assignment_target.name
 
 
 def is_mixed_case(name):
