@@ -23,36 +23,47 @@ def NotOK():
 def NotOK():
     pass
 #: Okay(--ignore-names=*OK)
+def NotOK():
+    pass
+#: N802:1:5(--ignore-names=*ok)
 def NotOK():
     pass
 #: Okay
 def _():
     pass
 #: Okay
-class Foo(object):
+class Foo:
     def __method(self):
         pass
 #: Okay
-class Foo(object):
+class Foo:
     def __method__(self):
         pass
 #: Okay
-class ClassName(object):
+class ClassName:
     def __method__(self):
         pass
 #: N802
-class ClassName(object):
+class ClassName:
     def notOk(self):
         pass
 #: Okay(--ignore-names=notOk)
-class ClassName(object):
+class ClassName:
     def notOk(self):
         pass
 #: Okay(--ignore-names=*Ok)
-class ClassName(object):
+class ClassName:
     def notOk(self):
         pass
 #: Okay
+class ClassName:
+    @override
+    def notOk(self):
+        pass
+    @typing.override
+    def alsoNotOk(self):
+        pass
+#: Okay
 def setUp():
     pass
 
@@ -60,6 +71,14 @@ def setUp():
 def tearDown():
     pass
 
+#: Okay
+def setUpModule():
+    pass
+
+#: Okay
+def tearDownModule():
+    pass
+
 #: Okay
 class TestCase:
     def setUp(self):
@@ -70,5 +89,25 @@ class TestCase:
         pass
     def tearDownClass(self):
         pass
+    def asyncSetUp(self):
+        pass
+    def asyncTearDown(self):
+        pass
     def setUpTestData(self):
         pass
+
+#: Okay
+def γ(x):
+    pass
+#: Okay
+def γ6(x):
+    pass
+
+#: Okay
+async def func(param1, param2):
+    do_stuff()
+    await some_coroutine()
+#: N802
+async def Func(param1, param2):
+    do_stuff()
+    await some_coroutine()
