@@ -12,26 +12,30 @@ class C:
     def m(cls, k='w'):  # noqa: N805
         pass
 #: N805
-class Foo(object):
+class Foo:
     def good(self, ads):
         pass
 
     def bad(ads, self):
         pass
 #: Okay(--ignore-names=a*)
-class Foo(object):
+class Foo:
     def bad(ads, self):
         pass
 #: Okay(--ignore-names=source)
-class GraphQLNode(object):
+class GraphQLNode:
+    def resolve_foo(source, info):
+        pass
+#: N805(--ignore-names=SOURCE)
+class GraphQLNode:
     def resolve_foo(source, info):
         pass
 #: Okay
-class Foo(object):
+class Foo:
     def __new__(cls):
         return object.__new__(Foo)
 #: Okay
-class Foo(object):
+class Foo:
     @classmethod
     def __prepare__(cls):
         pass
@@ -48,10 +52,18 @@ class Foo(object):
         pass
     test2 = staticmethod(test2)
 #: Okay
-class Foo(object):
+class Foo:
+    def __new__(cls):
+        pass
+#: Okay
+class Foo:
     def __init_subclass__(cls):
         pass
 #: Okay
+class Foo:
+    def __class_getitem__(cls, key):
+        pass
+#: Okay
 class Meta(type):
     def __new__(cls, name, bases, attrs):
         pass
@@ -70,7 +82,7 @@ class Meta(abc.ABCMeta):
     def test(cls):
         pass
 #: Okay(--classmethod-decorators=clazzy,cool)
-class NewClassmethodDecorators(object):
+class NewClassmethodDecorators:
     @clazzy
     def test1(cls, sy):
         pass
@@ -87,17 +99,45 @@ class NewClassmethodDecorators(object):
         pass
     test4 = cool(test4)
 #: N805(--classmethod-decorators=clazzy,cool)
-class ButWeLostTheOriginalClassMethodDecorator(object):
+class ButWeLostTheOriginalClassMethodDecorator:
     @classmethod
     def test(cls, sy):
         pass
 #: N805(--classmethod-decorators=clazzy,cool)
-class ButWeLostTheOriginalClassMethodLateDecorator(object):
+class ButWeLostTheOriginalClassMethodLateDecorator:
     def test(cls, sy):
         pass
     test = classmethod(test)
+#: Okay(--classmethod-decorators=myclassmethod)
+class C:
+    @myclassmethod('foo')
+    def bar(cls):
+        return 42
+#: Okay
+class PropertySetter:
+    @property
+    def var(self):
+        return True
+    @var.setter
+    def var(self, value):
+        self.var = value
+#: Okay
+class CalledInstanceDecorator:
+    @module.inner.decorator()
+    def test(self):
+        pass
+#: Okay(--classmethod-decorators=decorator)
+class CalledClassDecorator:
+    @module.inner.decorator()
+    def test(cls):
+        pass
+#: Okay(--staticmethod-decorators=decorator)
+class CalledStaticDecorator:
+    @module.inner.decorator()
+    def test():
+        pass
 #: Okay(--staticmethod-decorators=ecstatik,stcmthd)
-class NewStaticMethodDecorators(object):
+class NewStaticMethodDecorators:
     @ecstatik
     def test1(so, exciting):
         pass
@@ -114,12 +154,36 @@ class NewStaticMethodDecorators(object):
         pass
     test4 = stcmthd(test4)
 #: N805(--staticmethod-decorators=exstatik,stcmthd)
-class ButWeLostTheOriginalStaticMethodDecorator(object):
+class ButWeLostTheOriginalStaticMethodDecorator:
     @staticmethod
     def test(so, exciting):
         pass
 #: N805(--staticmethod-decorators=exstatik,stcmthd)
-class ButWeLostTheOriginalStaticMethodLateDecorator(object):
+class ButWeLostTheOriginalStaticMethodLateDecorator:
     def test(so, exciting):
         pass
     test = staticmethod(test)
+
+#: Okay
+class C:
+    async def __init__(*args, **kwargs):
+        pass
+#: N805:4:17
+class C:
+    @decorator(
+        'a')
+    async def m(cls, k='w'):  # noqa: N805
+        pass
+#: N805(--staticmethod-decorators=exstatik,stcmthd)
+class ButWeLostTheOriginalStaticMethodLateDecorator:
+    async def test(so, exciting):
+        pass
+    test = staticmethod(test)
+#: Okay
+class C:
+    def __init__(self, a, /, b=None):
+        pass
+#: N805:2:18
+class C:
+    def __init__(this, a, /, b=None):
+        pass
