@@ -8,7 +8,7 @@ def test():
 #: Okay
 GOOD = 1
 #: Okay
-class Test(object):
+class Test:
     GOOD = 1
 #: N806
 def test():
@@ -19,7 +19,7 @@ def test():
 #: N806
 def test():
     def test2():
-        class Foo(object):
+        class Foo:
             def test3(self):
                 Bad = 3
 #: Okay(--ignore-names=Bad)
@@ -29,6 +29,9 @@ def test():
 def test():
     Bad = 1
 #: Okay(--ignore-names=B*)
+def test():
+    Bad = 1
+#: N806(--ignore-names=b*)
 def test():
     Bad = 1
 #: Okay
@@ -175,3 +178,65 @@ def n():
 #: N806:2:26
 def e():
     return tuple(BaD for BaD in range(2))
+
+#: Okay
+VAR1, *VAR2, VAR3 = 1, 2, 3
+#: Okay
+Α, *Β, Γ = 1, 2, 3
+#: Okay
+[VAR1, *VAR2, VAR3] = (1, 2, 3)
+#: N806
+def extended_unpacking_not_ok():
+    Var1, *Var2, Var3 = 1, 2, 3
+#: N806
+def extended_unpacking_not_ok():
+    [Var1, *Var2, Var3] = (1, 2, 3)
+#: Okay
+def assing_to_unpack_ok():
+    a, *[b] = 1, 2
+#: N806
+def assing_to_unpack_not_ok():
+    a, *[bB] = 1, 2
+#: Okay
+Γ = 1
+#: N806
+def f():
+    Δ = 1
+#: N806
+def f():
+    _Δ = 1
+#: Okay
+def f():
+    γ = 1
+#: Okay
+def f():
+    _γ = 1
+#: Okay
+def f():
+    h, _, γ = s.partition('sep')
+
+#: Okay
+async def test():
+    good = 1
+#: N806
+async def f():
+    async with expr as ASYNC_VAR:
+        pass
+#: Okay
+def f1(values):
+    total = 0
+    partial_sums = [total := total + v for v in values]
+    return partial_sums, total
+#: Okay
+GLOBAL_VAR = 0
+def f2(values):
+    global GLOBAL_VAR
+    partial_sums = [GLOBAL_VAR := GLOBAL_VAR + v for v in values]
+    return partial_sums, GLOBAL_VAR
+#: N806:2:16
+def f():
+    return 1, (BaD_WalRuS := 1), BaD_WalRuS + 1
+#: Okay
+def f():
+    (NamedTuple := namedtuple('NamedTuple', 'f1 f2'))
+    return NamedTuple
