@@ -37,20 +37,23 @@ def bad__():
 def __bad__():
     pass
 #: Okay
-class ClassName(object):
+class ClassName:
     def method(self):
         def __bad():
             pass
 #: N807
-class ClassName(object):
+class ClassName:
     def method(self):
         def __bad__():
             pass
-#: Okay(--ignore-names=__bad)
-def __bad():
+#: Okay(--ignore-names=__bad__)
+def __bad__():
     pass
 #: Okay(--ignore-names=__*)
-def __bad():
+def __bad__():
+    pass
+#: N807(--ignore-names=__B*)
+def __bad__():
     pass
 #: Okay
 def __dir__():
@@ -58,3 +61,38 @@ def __dir__():
 #: Okay
 def __getattr__(name):
     pass
+
+#: Okay
+class C:
+    def γ(self):
+        pass
+#: Okay
+def __β(self):
+    pass
+#: Okay
+def β__(self):
+    pass
+#: N807
+def __β__(self):
+    pass
+#: N807
+def __β6__(self):
+    pass
+#: Okay
+class C:
+    def γ1(self):
+        pass
+
+#: Okay
+class C:
+    async def γ(self):
+        pass
+#: Okay
+async def __β(self):
+    pass
+#: Okay
+async def β__(self):
+    pass
+#: N807
+async def __β__(self):
+    pass
